--
-- Ion dock module configuration,
-- hacked to offer more control.
--

docktable=ioncore.read_savefile("dock_settings")

if docktable == nil then
    docktable = { mode="floating", pos="tl", grow="down" }
    ioncore.write_savefile("dock_settings", docktable)
end

-- Create a dock based on a saved settings file.
mod_dock.create{
    -- Dock mode: embedded|floating
    mode=docktable.mode,
    -- The screen to create the dock on
    screen=0,
    -- Corner or side of the screen to place the dock on.
    -- For embedded dock the valid values are: tl|tr|bl|br
    -- For floating dock the following are also valid: tc|bc|ml|mc|mr
    pos=docktable.pos,
    -- Growth direction: left|right|up|down
    grow=docktable.grow,
    -- Whether new dockapps should be added automatically to this dock
    is_auto=true,
    -- Show floating dock initially?
    floating_hidden=false,
    -- Name of the dock
    name="dock",
}


defbindings("WScreen", {
    bdoc("Toggle floating dock."),
    kpress(MOD1.."D", "mod_dock.set_floating_shown_on(_, 'toggle')"),

-- ___________
-- |1   2   3|   Mod1+K D (number) to move a floating dock.
-- |         |   Same for embedded dock, limited to 1,3,7,9.
-- |4   5   6|   
-- |         |   Also, Mod1+K D R for resize mode. Use ordinary move keys to move,
-- |7   8   9|   though the resize keys won't do very much.
-- -----------
    submap(MOD1.."K", {
	submap("D", {
	    kpress("1", "ioncore.region_list('WDock')[1]:set{pos='tl'}"),
	    kpress("2", "ioncore.region_list('WDock')[1]:set{pos='tc'}"),
	    kpress("3", "ioncore.region_list('WDock')[1]:set{pos='tr'}"),
	    kpress("4", "ioncore.region_list('WDock')[1]:set{pos='ml'}"),
	    kpress("5", "ioncore.region_list('WDock')[1]:set{pos='mc'}"),
	    kpress("6", "ioncore.region_list('WDock')[1]:set{pos='mr'}"),
	    kpress("7", "ioncore.region_list('WDock')[1]:set{pos='bl'}"),
	    kpress("8", "ioncore.region_list('WDock')[1]:set{pos='bc'}"),
	    kpress("9", "ioncore.region_list('WDock')[1]:set{pos='br'}"),
	    
	    kpress("H", "ioncore.region_list('WDock')[1]:set{grow='left'}"),
	    kpress("J", "ioncore.region_list('WDock')[1]:set{grow='down'}"),
	    kpress("K", "ioncore.region_list('WDock')[1]:set{grow='up'}"),
	    kpress("L", "ioncore.region_list('WDock')[1]:set{grow='right'}"),

	    kpress("R", "ioncore.region_list('WDock')[1]:begin_kbresize()"),
	    kpress("F", "flip_dock_direction()"),

	    -- Mod1+K D E toggles between floating and embedded dock modes.
	    -- Doing this restarts ion, so don't use it if you've got
	    -- config files open, etc.
	    kpress("E", "swap_dockmodes()"),
	}),
    }),
})

-- Dock settings menu. For this to work, mod_menu must have been loaded 
-- previously.
if mod_menu then
    if docktable.mode=="floating" then
        defmenu("dock-settings", {
	    menuentry("Hide dock", "mod_dock.set_floating_shown_on("..
	    	"_:screen_of(), 'toggle')"),
            menuentry("Pos-TL", "_:set{pos='tl'}"),
            menuentry("Pos-TC", "_:set{pos='tc'}"),
            menuentry("Pos-TR", "_:set{pos='tr'}"),
            menuentry("Pos-ML", "_:set{pos='ml'}"),
            menuentry("Pos-MC", "_:set{pos='mc'}"),
            menuentry("Pos-MR", "_:set{pos='mr'}"),
            menuentry("Pos-BL", "_:set{pos='bl'}"),
            menuentry("Pos-BC", "_:set{pos='bc'}"),
            menuentry("Pos-BR", "_:set{pos='br'}"),
    
            menuentry("Grow-L", "_:set{grow='left'}"),
            menuentry("Grow-R", "_:set{grow='right'}"),
            menuentry("Grow-U", "_:set{grow='up'}"),
            menuentry("Grow-D", "_:set{grow='down'}"),
    	    menuentry("Embed dock", "swap_dockmodes()"),
        })
    else
        defmenu("dock-settings", {
            menuentry("Pos-TL", "_:set{pos='tl'}"),
            menuentry("Pos-TR", "_:set{pos='tr'}"),
            menuentry("Pos-BL", "_:set{pos='bl'}"),
            menuentry("Pos-BR", "_:set{pos='br'}"),
    
            menuentry("Grow-L", "_:set{grow='left'}"),
            menuentry("Grow-R", "_:set{grow='right'}"),
            menuentry("Grow-U", "_:set{grow='up'}"),
            menuentry("Grow-D", "_:set{grow='down'}"),
    	    menuentry("Float dock", "swap_dockmodes()"),
        })
    end

    -- To use any of the dock's mouse bindings,
    -- you'll have to find a bit of dead space for clicking in.
    -- Near the border should always work.
    -- (This is even necessary for the MOD1.. bindings.)
    defbindings("WDock", {
        mpress("Button3", "mod_menu.pmenu(_, _sub, 'dock-settings')"),
        mpress("Button2", "cycle_dock_direction()"),
        mclick("Button1", "flip_dock_direction()"),
	
	-- You can actually drag a floating dock anywhere.
	-- Position will be lost when ion restarts.
	mdrag("Button1", "_:p_move()"),
	mdrag(MOD1.."Button1", "_:p_move()"),
    })
end

function flip_dock_direction()
    currentdock = ioncore.region_list('WDock')[1]
    dockget = currentdock:get()
    if dockget.grow=="left" then
    	currentdock:set{grow='right'}
    elseif dockget.grow=="right" then
    	currentdock:set{grow='left'}
    elseif dockget.grow=="up" then
    	currentdock:set{grow='down'}
    elseif dockget.grow=="down" then
    	currentdock:set{grow='up'}
    end
end

function cycle_dock_direction()
    currentdock = ioncore.region_list('WDock')[1]
    dockget = currentdock:get()
    if dockget.grow=="left" then
    	currentdock:set{grow='up'}
    elseif dockget.grow=="right" then
    	currentdock:set{grow='down'}
    elseif dockget.grow=="up" then
    	currentdock:set{grow='right'}
    elseif dockget.grow=="down" then
    	currentdock:set{grow='left'}
    end
end

function save_dock_settings()
    dockget = ioncore.region_list('WDock')[1]:get()
    docktable.grow = dockget.grow
    if docktable.mode=="embedded" then
    	docktable.pos=string.gsub(string.gsub(dockget.pos, "m", "t"), "c", "l")
    else
    	docktable.pos = dockget.pos
    end
    ioncore.write_savefile("dock_settings", docktable)
end

function swap_dockmodes()
   if docktable.mode=="embedded" then
   	docktable.mode="floating"
   else
   	docktable.mode="embedded"
   end
   save_dock_settings()
   ioncore.restart()
end

--
-- This makes sure dock settings are saved any time ion restarts.
-- 
ioncore.get_hook("ioncore_deinit_hook"):add(
    function() save_dock_settings() end
)


--
-- To show a hidden floating dock using the mouse, I've put this in my 
-- context menu:
--
-- menuentry("Show Dock", "mod_dock.set_floating_shown_on(_:screen_of(), 'toggle')"),
